#ifndef BUTTON_DEBOUNCER_H
#define BUTTON_DEBOUNCER_H

#include <cstdint>
#include <ctime>
#include <chrono>
#include <tuple>

/*
 * Usage: create class instance and call method syncButtons with the buttons field as parameter (or
 * any vector representing the pressed buttons). To check if a button was pressed call method
 * buttonPressed with the index of the button as parameter. Minimum elapsed time in ms since last
 * press and supported number of buttons can be passed through constructor.
 */

class ButtonDebouncer
{

public:
  ButtonDebouncer(double min_time = 200, int n_buttons = 12)
    : number_of_buttons_(n_buttons), min_time_(min_time)
  {
    buttons_.reserve(number_of_buttons_);
    auto now = std::chrono::high_resolution_clock::now();
    for(int i = 0; i < number_of_buttons_; ++i) {
      buttons_.push_back(std::make_tuple(0, now, now));
    }
  }

  void syncButtons(const std::vector<int32_t> values)
  {
    number_of_buttons_ = values.size();
    auto now = std::chrono::high_resolution_clock::now();
    for(int i = 0; i < number_of_buttons_; ++i) {
      if(values[i] == BUTTON_PRESSED) {
        auto last_press = std::get<1>(buttons_[i]);
        buttons_[i] = std::make_tuple(values[i], now, last_press);
      } else {
        std::get<0>(buttons_[i]) = 0;
      }
    }
  }

  bool buttonPressed(const int index)
  {
    return (std::get<0>(buttons_[index]) == BUTTON_PRESSED) && isLongAgo(index);
  }

private:
  bool isLongAgo(const int index)
  {
    auto elapsed_time_ms = std::chrono::duration<double, std::milli>(std::get<1>(buttons_[index]) -
          std::get<2>(buttons_[index])).count();
    return elapsed_time_ms > min_time_;
  }

  const int32_t BUTTON_PRESSED = 1; /*!< Value for a pressed button. */
  size_t number_of_buttons_;
  std::vector<std::tuple<int32_t, std::chrono::time_point<std::chrono::high_resolution_clock>,
                         std::chrono::time_point<std::chrono::high_resolution_clock>>>
      buttons_;
  double min_time_;
};

#endif // BUTTON_DEBOUNCER_H
